`timescale 1ns / 1ps

// 4 digites 7 szegmenses vezrlo
module cntrl_seg7(
   input  wire       clk,     // rajel
   input  wire       rst,     // Reset jel

   input  wire [3:0] d0,      // 0. digiten megjelentend rtk
   input  wire [3:0] d1,      // 1. digiten megjelentend rtk
   input  wire [3:0] d2,      // 2. digiten megjelentend rtk
   input  wire [3:0] d3,      // 3. digiten megjelentend rtk
   
   output wire [3:0] dig_n,   // Digit engedlyezo jel
   output wire [7:0] seg_n    // Szegmens jelek
);


// ~ kHz frekvencij engedlyezo jel: 16 MHz rajel --> ~16000 rajel peridusideju
// Eroforrs takarkossgra trekvs miatt legyen 16385 rajel peridusideju
reg [14:0] cntr;  // Szmll az engedlyezo jel generlshoz
wire ce; // ~kHz frekvencij engedlyezo jel

always @ (posedge clk)
if (rst)
   cntr <= 15'b0;
else if (ce)
   cntr <= 15'b0;
else
   cntr <= cntr + 1;

assign ce = cntr[14];

// 4 bites, reset-elheto, visszacsatolt shiftregiszter
//   a digit engedlyezo jel generlshoz
reg [3:0] shr;
always @ (posedge clk)
if (rst)
   shr <= 4'b1110;
else if (ce)
   shr <= {shr[2:0], shr[3]};
   
assign dig_n = shr;

// 2 bites szmll, szinkronban jr a digit engedlyezo shift regiszterrel
// Mindig a kivlasztott digit sorszmt mutatja
reg [1:0] dig_cntr;
always @ (posedge clk)
if (rst)
   dig_cntr <= 2'b0;
else if (ce)
   dig_cntr <= dig_cntr + 1;


// 4 bites, 4:1 multiplexer az adat kivlasztshoz
reg [3:0] mux;
always @ ( * )
case (dig_cntr)
   2'b01:   mux <= d1;
   2'b10:   mux <= d2;
   2'b11:   mux <= d3;
   default: mux <= d0;
endcase

// binris -> szegmens talakt
//      0
//     ---
//  5 |   | 1
//     --- <--6
//  4 |   | 2
//     ---
//      3

reg [7:0] seg;

always @( * )
case (mux)
   4'b0001 : seg <= 8'b11111001;   // 1
   4'b0010 : seg <= 8'b10100100;   // 2
   4'b0011 : seg <= 8'b10110000;   // 3
   4'b0100 : seg <= 8'b10011001;   // 4
   4'b0101 : seg <= 8'b10010010;   // 5
   4'b0110 : seg <= 8'b10000010;   // 6
   4'b0111 : seg <= 8'b11111000;   // 7
   4'b1000 : seg <= 8'b10000000;   // 8
   4'b1001 : seg <= 8'b10010000;   // 9
   4'b1010 : seg <= 8'b10001000;   // A
   4'b1011 : seg <= 8'b10000011;   // b
   4'b1100 : seg <= 8'b11000110;   // C
   4'b1101 : seg <= 8'b10100001;   // d
   4'b1110 : seg <= 8'b10000110;   // E
   4'b1111 : seg <= 8'b10001110;   // F
   default : seg <= 8'b11000000;   // 0
endcase


assign seg_n = seg;


endmodule
